@@ -11,9 +11,11 @@ from abc import ABC
 from collections.abc import Callable, Mapping
 from typing import Any, TypeVar, cast
 
+from roborock.callbacks import CallbackList
 from roborock.data import HomeDataDevice, HomeDataProduct
 from roborock.exceptions import RoborockException
 from roborock.roborock_message import RoborockMessage
+from roborock.util import RoborockLoggerAdapter
 
 from .channel import Channel
 from .traits import Trait
@@ -22,6 +24,7 @@ from .traits.traits_mixin import TraitsMixin
 _LOGGER = logging.getLogger(__name__)
 
 __all__ = [
+    "DeviceReadyCallback",
     "RoborockDevice",
 ]
 
@@ -32,6 +35,9 @@ BACKOFF_MULTIPLIER = 1.5
 START_ATTEMPT_TIMEOUT = datetime.timedelta(seconds=5)
 
 
+DeviceReadyCallback = Callable[["RoborockDevice"], None]
+
+
 class RoborockDevice(ABC, TraitsMixin):
     """A generic channel for establishing a connection with a Roborock device.
 
@@ -59,12 +65,15 @@ class RoborockDevice(ABC, TraitsMixin):
         """
         TraitsMixin.__init__(self, trait)
         self._duid = device_info.duid
+        self._logger = RoborockLoggerAdapter(self._duid, _LOGGER)
         self._name = device_info.name
         self._device_info = device_info
         self._product = product
         self._channel = channel
         self._connect_task: asyncio.Task[None] | None = None
         self._unsub: Callable[[], None] | None = None
+        self._ready_callbacks = CallbackList["RoborockDevice"]()
+        self._has_connected = False
 
     @property
     def duid(self) -> str:
@@ -108,6 +117,22 @@ class RoborockDevice(ABC, TraitsMixin):
         """
         return self._channel.is_local_connected
 
+    def add_ready_callback(self, callback: DeviceReadyCallback) -> Callable[[], None]:
+        """Add a callback to be notified when the device is ready.
+
+        A device is considered ready when it has successfully connected. It may go
+        offline later, but this callback will only be called once when the device
+        first connects.
+
+        The callback will be called immediately if the device has already previously
+        connected.
+        """
+        remove = self._ready_callbacks.add_callback(callback)
+        if self._has_connected:
+            callback(self)
+
+        return remove
+
     async def start_connect(self) -> None:
         """Start a background task to connect to the device.
 
@@ -131,17 +156,16 @@ class RoborockDevice(ABC, TraitsMixin):
                     try:
                         await self.connect()
                         start_attempt.set()
+                        self._has_connected = True
+                        self._ready_callbacks(self)
                         return
                     except RoborockException as e:
                         start_attempt.set()
-                        _LOGGER.info("Failed to connect to device %s: %s", self.name, e)
-                        _LOGGER.info(
-                            "Retrying connection to device %s in %s seconds", self.name, backoff.total_seconds()
-                        )
+                        self._logger.info("Failed to connect (retry %s): %s", backoff.total_seconds(), e)
                         await asyncio.sleep(backoff.total_seconds())
                         backoff = min(backoff * BACKOFF_MULTIPLIER, MAX_BACKOFF_INTERVAL)
             except asyncio.CancelledError:
-                _LOGGER.info("connect_loop for device %s was cancelled", self.name)
+                self._logger.debug("connect_loop was cancelled for device %s", self.duid)
                 # Clean exit on cancellation
                 return
             finally:
@@ -152,14 +176,13 @@ class RoborockDevice(ABC, TraitsMixin):
         try:
             await asyncio.wait_for(start_attempt.wait(), timeout=START_ATTEMPT_TIMEOUT.total_seconds())
         except TimeoutError:
-            _LOGGER.debug("Initial connection attempt to device %s is taking longer than expected", self.name)
+            self._logger.debug("Initial connection attempt took longer than expected, will keep trying in background")
 
     async def connect(self) -> None:
         """Connect to the device using the appropriate protocol channel."""
         if self._unsub:
             raise ValueError("Already connected to the device")
         unsub = await self._channel.subscribe(self._on_message)
-        _LOGGER.info("Connected to V1 device %s", self.name)
         if self.v1_properties is not None:
             try:
                 await self.v1_properties.discover_features()
@@ -182,7 +205,7 @@ class RoborockDevice(ABC, TraitsMixin):
 
     def _on_message(self, message: RoborockMessage) -> None:
         """Handle incoming messages from the device."""
-        _LOGGER.debug("Received message from device: %s", message)
+        self._logger.debug("Received message from device: %s", message)
 
     def diagnostic_data(self) -> dict[str, Any]:
         """Return diagnostics information about the device."""
@@ -198,7 +221,23 @@ class RoborockDevice(ABC, TraitsMixin):
 
 T = TypeVar("T")
 
-REDACT_KEYS = {"duid", "localKey", "mac", "bssid", "sn", "ip"}
+REDACT_KEYS = {
+    # Potential identifiers
+    "duid",
+    "localKey",
+    "mac",
+    "bssid",
+    "sn",
+    "ip",
+    "u",
+    "s",
+    "h",
+    "k",
+    # Large binary blobs are entirely omitted
+    "imageContent",
+    "mapData",
+    "rawApiResponse",
+}
 REDACTED = "**REDACTED**"
 
 
